/*
  C function similar to the Matlab program test_optim_vmlmb_const.m
  to test the Matlab interface to the OptimPack optimisation Package
  for a unconstrained minimization of the rosenbrock function

  The results are identical to the test_optim_vmlmb_const.m ones.

  Compilation with the OptimPack library:
  gcc -o test_optim_vmlmb_const.o -c test_optim_vmlmb_const.c -Wall
  gcc -o test_optim_vmlmb_const test_optim_vmlmb_const.o -lm -loptimpack
  ./test_optim_vmlmb_const

*/
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include "optimpack.h"

/* Compute the rosenbrock function */
double funct(double *x)
{
  double f;
    f=(1.0-x[0])*(1.0-x[0])+100.0*((x[1]-x[0]*x[0])*(x[1]-x[0]*x[0]));
  return f;                
}
            
/* Compute the gradient of the rosenbrock function */
void grad(double *x,double *gr)
{
  gr[0] = -2.0*(1.0-x[0]) - 400.0*(x[1]-x[0]*x[0])*x[0];
  gr[1] = 200.0*(x[1]-x[0]*x[0]);     
}
                         
int main()
{
  double x[2]={-1.0,1.0}, f,g[2], *dsave, xmin[2]={-1.0,0.0}, xmax[2]={1.0,0.75} ;
  double fold, normg;
  double fatol=0.0, frtol=1e-8, sftol=0.001, sgtol=0.9, sxtol=0.1, epsilon=0.01, costheta=0.4; 
  char *csave;
  int task, n=2, m=3, iter, nbitermax, nbeval=0, k;
  op_integer_t *isave, active[2];

  printf("Minimization of the Rosenbrok function : n = %d, m=%d\n",n,m);
  printf("Initialization x= [%f ; %f]\n",x[0],x[1]);
  printf("Lower bounds xmin = [%f ; %f]\n",xmin[0],xmin[1]);
  printf("Upper bounds xmax = [%f ; %f]\n",xmax[0],xmax[1]);
  // Allocate and setup workspaces:
  //  printf("Memory allocation for csave : size %d\n",OP_VMLMB_CSAVE_NUMBER);
  //  printf("Memory allocation for isave : size %d\n",OP_VMLMB_ISAVE_NUMBER);
  //  printf("Memory allocation for dsave : size %d\n",OP_VMLMB_DSAVE_NUMBER(n,m));
  csave = malloc(OP_VMLMB_CSAVE_NUMBER*sizeof(char));
  isave = malloc(OP_VMLMB_ISAVE_NUMBER*sizeof(op_integer_t));
  dsave = malloc(OP_VMLMB_DSAVE_NUMBER(n, m)*sizeof(double));

  task = op_vmlmb_first(n,m,fatol,frtol,sftol,sgtol,sxtol,epsilon,costheta,csave,isave,dsave);
  //  printf("isave(2:11) = [ ");
  //  for (k=2;k<11;k++)
  //      printf("%d, ",isave[k]);printf("]\n");
  //  printf("dsave(12:26) = [ ");
  //  for (k=12;k<26;k++)
  //      printf("%e, ",dsave[k]);printf("]\n");

  printf("it\tnbeval\t  x\t\t\t  f\tdf\tnormg\ttask\tstage\n");
  nbitermax=1000;f=funct(x)+1e8;  
  for (iter=1 ; iter<=nbitermax ;iter++)
  {
      if (task == OP_TASK_FG)
      {
        /* apply bound constraints */
          for (k=0;k<n;k++) {
              if (x[k] < xmin[k])
                x[k] = xmin[k];
              else if (x[k] > xmax[k])
                x[k] = xmax[k];
          }
        fold = f;
        f=funct(x);        // evaluate the function at X; store in F
        grad(x,g);         // evaluate the gradient of F at X; store in G	
        nbeval++;          // nb evaluation of function and gradient
        normg = 0.0;       // norm of the gradient
        for (k=0;k<n;k++)
            normg += g[k]*g[k];
      } else if (task == OP_TASK_NEWX) 
      {
        // New successful step: the approximation X, function F, and
        // gradient G, are available for inspection.
        // printf("Successfull step inspection : \n");
      } else {
        // Convergence, or error, or warning
        //	printf("Convergence, or error, or warning :\n");
    	break;
      }
        // Computes set of active parameters:
      if (iter == 1 || task == OP_TASK_NEWX) {
          for (k=0;k<n;k++) {
            if ((x[k] > xmin[k] || g[k] < 0) && (x[k] < xmax[k] || g[k] > 0))
                active[k] = 1;
            else
                active[k] = 0;
          }
      }
      // Computes next step:
      task = op_vmlmb_next(x, &f, g, active, NULL, csave, isave, dsave);
      printf("%d\t%d\t[%7.2e ; %7.2e]\t%6.2g\t%6.2g\t%6.2g\t%d\t%d\n",iter,nbeval,x[0],x[1],f,fold-f,normg,task,isave[3]);
    
  }
  printf("%s\n", csave);
  printf("function = %.8f, grad = [%.8f ; %.8f]\n" , f, g[0], g[1]);
  printf("Solution : x= [%.8f ; %.8f]\n", x[0],x[1]);
  return 1;
}
